@@ -1,4 +1,4 @@
-# Copyright 2004-2015 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2016 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -88,8 +88,8 @@ class ScreenProfile(renpy.object.Object):
             Displays the variables in the screen that are marked as const and
             not-const.
 
-        All profiling profiling output will be logged to profile_screen.txt in
-        the game directory.
+        All profiling output will be logged to profile_screen.txt in the game
+        directory.
         """
 
         self.predict = predict
@@ -199,7 +199,8 @@ class Screen(renpy.object.Object):
                  predict=None,
                  variant=None,
                  parameters=False,
-                 location=None):
+                 location=None,
+                 layer="screens"):
 
         # The name of this screen.
         if isinstance(name, basestring):
@@ -207,8 +208,12 @@ class Screen(renpy.object.Object):
 
         self.name = name
 
-        screens[name[0], variant] = self
-        screens_by_name[name[0]][variant] = self
+        if (variant is None) or isinstance(variant, basestring):
+            variant = [ variant ]
+
+        for v in variant:
+            screens[name[0], v] = self
+            screens_by_name[name[0]][v] = self
 
         # A function that can be called to display the screen.
         self.function = function
@@ -241,6 +246,9 @@ class Screen(renpy.object.Object):
         # The location (filename, linenumber) of this screen.
         self.location = location
 
+        # The layer the screen will be shown on.
+        self.layer = layer
+
         global prepared
         global analyzed
 
@@ -871,28 +879,62 @@ def define_screen(*args, **kwargs):
     Screen(*args, **kwargs)
 
 
-def get_screen(name, layer="screens"):
+def get_screen_layer(name):
+    """
+    Returns the layer that the screen with `name` is part of.
+    """
+
+    if not isinstance(name, basestring):
+        name = name[0]
+
+    screen = get_screen_variant(name)
+
+    if screen is None:
+        return "screens"
+    else:
+        return screen.layer
+
+
+def get_screen(name, layer=None):
     """
     :doc: screens
 
-    Returns the ScreenDisplayable with the given `tag`, on
-    `layer`. If no displayable with the tag is not found, it is
-    interpreted as screen name. If it's still not found, None is returned.
-     """
+    Returns the ScreenDisplayable with the given `name` on layer. `name`
+    is first interpreted as a tag name, and then a screen name. If the
+    screen is not showing, returns None.
+
+    This can also take a list of names, in which case the first screen
+    that is showing is returned.
+
+    This function can be used to check if a screen is showing, using code
+    like::
+
+        if renpy.get_screen("say"):
+            text "The say screen is showing."
+        else:
+            text "The say screen is hidden."
+
+    """
+
+    if layer is None:
+        layer = get_screen_layer(name)
 
     if isinstance(name, basestring):
-        name = tuple(name.split())
-
-    tag = name[0]
+        name = (name, )
 
     sl = renpy.exports.scene_lists()
 
-    sd = sl.get_displayable_by_tag(layer, tag)
+    for tag in name:
+
+        sd = sl.get_displayable_by_tag(layer, tag)
+        if sd is not None:
+            return sd
 
-    if sd is None:
         sd = sl.get_displayable_by_name(layer, name)
+        if sd is not None:
+            return sd
 
-    return sd
+    return None
 
 
 def has_screen(name):
@@ -939,7 +981,7 @@ def show_screen(_screen_name, *_args, **kwargs):
     initialize the screen's scope.
     """
 
-    _layer = kwargs.pop("_layer", "screens")
+    _layer = kwargs.pop("_layer", None)
     _tag = kwargs.pop("_tag", None)
     _widget_properties = kwargs.pop("_widget_properties", {})
     _transient = kwargs.pop("_transient", False)
@@ -954,6 +996,9 @@ def show_screen(_screen_name, *_args, **kwargs):
     if screen is None:
         raise Exception("Screen %s is not known.\n" % (name[0],))
 
+    if _layer is None:
+        _layer = get_screen_layer(name)
+
     if _tag is None:
         _tag = screen.tag
 
@@ -996,7 +1041,7 @@ def predict_screen(_screen_name, *_args, **kwargs):
     initialize the screen's scope.
     """
 
-    _layer = kwargs.pop("_layer", "screens")
+    _layer = kwargs.pop("_layer", None)
     _tag = kwargs.pop("_tag", None)
     _widget_properties = kwargs.pop("_widget_properties", {})
     _transient = kwargs.pop("_transient", False)
@@ -1014,6 +1059,9 @@ def predict_screen(_screen_name, *_args, **kwargs):
     if screen is None:
         return
 
+    if _layer is None:
+        _layer = get_screen_layer(name)
+
     scope = { }
     scope["_scope"] = scope
 
@@ -1048,7 +1096,7 @@ def predict_screen(_screen_name, *_args, **kwargs):
     renpy.ui.reset()
 
 
-def hide_screen(tag, layer='screens'):
+def hide_screen(tag, layer=None):
     """
     :doc: screens
 
@@ -1057,6 +1105,9 @@ def hide_screen(tag, layer='screens'):
     Hides the screen with `tag` on `layer`.
     """
 
+    if layer is None:
+        layer = get_screen_layer((tag,))
+
     screen = get_screen(tag, layer)
 
     if screen is not None:
@@ -1098,7 +1149,7 @@ def use_screen(_screen_name, *_args, **kwargs):
 def current_screen():
     return _current_screen
 
-def get_widget(screen, id, layer='screens'): #@ReservedAssignment
+def get_widget(screen, id, layer=None): #@ReservedAssignment
     """
     :doc: screens
 
@@ -1113,6 +1164,9 @@ def get_widget(screen, id, layer='screens'): #@ReservedAssignment
     if screen is None:
         screen = current_screen()
     else:
+        if layer is None:
+            layer = get_screen_layer(screen)
+
         screen = get_screen(screen, layer)
 
     if not isinstance(screen, ScreenDisplayable):
@@ -1124,7 +1178,7 @@ def get_widget(screen, id, layer='screens'): #@ReservedAssignment
     rv = screen.widgets.get(id, None)
     return rv
 
-def get_widget_properties(id, screen=None, layer='screens'): # @ReservedAssignment
+def get_widget_properties(id, screen=None, layer=None): # @ReservedAssignment
     """
     :doc: screens
 
@@ -1141,6 +1195,9 @@ def get_widget_properties(id, screen=None, layer='screens'): # @ReservedAssignme
     if screen is None:
         s = current_screen()
     else:
+        if layer is None:
+            layer = get_screen_layer(screen)
+
         s = get_screen(screen, layer)
 
     if s is None:
@@ -1168,3 +1225,29 @@ def before_restart():
             if isinstance(i, ScreenDisplayable):
                 i.restarting = True
 
+
+def show_overlay_screens(suppress_overlay):
+    """
+    Called from interact to show or hide the overlay screens.
+    """
+
+    show = not suppress_overlay
+
+    if renpy.store._overlay_screens is None:
+        show = show
+    elif renpy.store._overlay_screens is True:
+        show = True
+    else:
+        show = False
+
+    if show:
+
+        for i in renpy.config.overlay_screens:
+            if get_screen(i) is None:
+                show_screen(i)
+
+    else:
+
+        for i in renpy.config.overlay_screens:
+            if get_screen(i) is not None:
+                hide_screen(i)
