@@ -1,4 +1,4 @@
-# Copyright 2004-2015 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2016 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -27,6 +27,8 @@
 # Remember the real file.
 _file = file
 
+import re
+
 import renpy.display
 import renpy.audio
 
@@ -50,8 +52,9 @@ from renpy.text.extras import ParameterizedText
 from renpy.text.font import register_sfont, register_mudgefont, register_bmfont
 from renpy.text.text import language_tailor
 from renpy.display.behavior import Keymap
-from renpy.display.behavior import run as run_action, run_unhovered, run_periodic
+from renpy.display.behavior import run, run as run_action, run_unhovered, run_periodic
 from renpy.display.behavior import map_event, queue_event, clear_keymap_cache
+from renpy.display.behavior import is_selected, is_sensitive
 
 from renpy.display.minigame import Minigame
 from renpy.display.screen import define_screen, show_screen, hide_screen, use_screen, current_screen
@@ -60,11 +63,13 @@ from renpy.display.screen import get_widget_properties
 
 from renpy.display.focus import focus_coordinates
 from renpy.display.predict import screen as predict_screen
+
 from renpy.display.image import image_exists, image_exists as has_image
+from renpy.display.image import get_available_image_tags, get_available_image_attributes
+
 from renpy.display.im import load_surface, load_image
 
 from renpy.curry import curry, partial
-from renpy.audio.sound import play
 from renpy.display.video import movie_start_fullscreen, movie_start_displayable, movie_stop
 
 from renpy.loadsave import load, save, list_saved_games, can_load, rename_save, copy_save, unlink_save, scan_saved_game
@@ -120,7 +125,7 @@ def public_api():
     ParameterizedText
     register_sfont, register_mudgefont, register_bmfont
     Keymap
-    run_action, run_unhovered, run_periodic, map_event
+    run, run_action, run_unhovered, run_periodic, map_event
     Minigame
     curry, partial
     play
@@ -148,6 +153,7 @@ def public_api():
     map_event, queue_event, clear_keymap_cache
     const, pure, not_const
     image_exists, has_image
+    get_available_image_tags, get_available_image_attributes
     load_image, load_surface
     profile_memory, diff_memory, profile_rollback
     TEXT_TAG
@@ -157,6 +163,7 @@ def public_api():
     not_infinite_loop
     register_sl_statement, register_sl_displayable
     eval_who
+    is_selected, is_sensitive
 
 del public_api
 
@@ -375,6 +382,61 @@ def copy_images(old, new):
             renpy.display.image.register_image(new + k[lenold:], v)
 
 
+def default_layer(layer, tag, expression=False):
+    """
+    :undocumented:
+
+    If layer is not None, returns it. Otherwise, interprets `tag` as a name
+    or tag, then looks up what the default layer for that tag is, and returns
+    the result.
+    """
+
+    if layer is not None:
+        return layer
+
+    if expression:
+        return 'master'
+
+    if isinstance(tag, tuple):
+        tag = tag[0]
+    elif " " in tag:
+        tag = tag.split()[0]
+
+    return renpy.config.tag_layer.get(tag, renpy.config.default_tag_layer)
+
+
+def can_show(name, layer=None, tag=None):
+    """
+    :doc: image_func
+
+    Determines if `name` can be used to show an image. This interprets `name`
+    as a tag and attributes. This is combined with the attributes of the
+    currently-showing image with `tag` on `layer` to try to determine a unique image
+    to show. If a unique image can be show, returns the name of that image as
+    a tuple. Otherwise, returns None.
+
+    `tag`
+        The image tag to get attributes from. If not given, defaults to the first
+        component of `name`.
+
+    `layer`
+        The layer to check. If None, uses the default layer for `tag`.
+    """
+
+    if not isinstance(name, tuple):
+        name = tuple(name.split())
+
+    if tag is None:
+        tag = name[0]
+
+    layer = default_layer(layer, None)
+
+    try:
+        return renpy.game.context().images.apply_attributes(layer, tag, name)
+    except:
+        return None
+
+
 def showing(name, layer='master'):
     """
     :doc: image_func
@@ -386,11 +448,17 @@ def showing(name, layer='master'):
         May be a string giving the image name or a tuple giving each
         component of the image name. It may also be a string giving
         only the image tag.
+
+
+    `layer`
+        The layer to check. If None, uses the default layer for `tag`.
     """
 
     if not isinstance(name, tuple):
         name = tuple(name.split())
 
+    layer = default_layer(layer, name)
+
     return renpy.game.context().images.showing(layer, name)
 
 def get_showing_tags(layer='master'):
@@ -402,7 +470,7 @@ def get_showing_tags(layer='master'):
 
     return renpy.game.context().images.get_showing_tags(layer)
 
-def predict_show(name, layer='master', what=None, tag=None, at_list=[ ]):
+def predict_show(name, layer=None, what=None, tag=None, at_list=[ ]):
     """
     :undocumented:
 
@@ -412,7 +480,7 @@ def predict_show(name, layer='master', what=None, tag=None, at_list=[ ]):
         The name of the image to show, a string.
 
     `layer`
-        The layer the image is being show non.
+        The layer the image is being shown on.
 
     `what`
         What is being show - if given, overrides `name`.
@@ -426,6 +494,8 @@ def predict_show(name, layer='master', what=None, tag=None, at_list=[ ]):
 
     key = tag or name[0]
 
+    layer = default_layer(key, layer)
+
     if what is None:
         what = name
     elif isinstance(what, basestring):
@@ -457,7 +527,7 @@ def predict_show(name, layer='master', what=None, tag=None, at_list=[ ]):
     renpy.display.predict.displayable(img)
 
 
-def show(name, at_list=[ ], layer='master', what=None, zorder=None, tag=None, behind=[ ], atl=None, transient=False, munge_name=True):
+def show(name, at_list=[ ], layer=None, what=None, zorder=None, tag=None, behind=[ ], atl=None, transient=False, munge_name=True):
     """
     :doc: se_images
     :args: (name, at_list=[ ], layer='master', what=None, zorder=0, tag=None, behind=[ ])
@@ -474,7 +544,8 @@ def show(name, at_list=[ ], layer='master', what=None, zorder=None, tag=None, be
 
     `layer`
         A string, giving the name of the layer on which the image will be shown.
-        The equivalent of the ``onlayer`` property.
+        The equivalent of the ``onlayer`` property. If None, uses the default
+        layer associated with the tag.
 
     `what`
         If not None, this is a displayable that will be shown in lieu of
@@ -487,7 +558,7 @@ def show(name, at_list=[ ], layer='master', what=None, zorder=None, tag=None, be
         zorder is preserved if it exists, and is otherwise set to 0.
 
     `tag`
-        A string, used to specify the the image tag of the shown image. The
+        A string, used to specify the image tag of the shown image. The
         equivalent of the ``as`` property.
 
     `behind`
@@ -509,10 +580,20 @@ def show(name, at_list=[ ], layer='master', what=None, zorder=None, tag=None, be
     sls = scene_lists()
     key = tag or name[0]
 
+    layer = default_layer(layer, key)
+
     if renpy.config.sticky_positions:
         if not at_list and key in sls.at_list[layer]:
             at_list = sls.at_list[layer][key]
 
+    if not at_list:
+        tt = renpy.config.tag_transform.get(key, None)
+        if tt is not None:
+            if not isinstance(tt, list):
+                at_list = [ tt ]
+            else:
+                at_list = list(tt)
+
     if what is None:
         what = name
     elif isinstance(what, basestring):
@@ -566,7 +647,7 @@ def show(name, at_list=[ ], layer='master', what=None, zorder=None, tag=None, be
     sls.add(layer, img, key, zorder, behind, at_list=at_list, name=name, atl=atl, default_transform=default_transform, transient=transient)
 
 
-def hide(name, layer='master'):
+def hide(name, layer=None):
     """
     :doc: se_images
 
@@ -577,7 +658,8 @@ def hide(name, layer='master'):
          any image with the tag is hidden (the precise name does not matter).
 
     `layer`
-         The layer on which this function operates.
+         The layer on which this function operates. If None, uses the default
+         layer associated with the tag.
     """
 
     if renpy.game.context().init_phase:
@@ -588,6 +670,9 @@ def hide(name, layer='master'):
 
     sls = scene_lists()
     key = name[0]
+
+    layer = default_layer(layer, key)
+
     sls.remove(layer, key)
 
     if renpy.config.missing_hide:
@@ -612,6 +697,9 @@ def scene(layer='master'):
         $ renpy.show("bg beach")
     """
 
+    if layer is None:
+        layer = 'master'
+
     if renpy.game.context().init_phase:
         raise Exception("Scene may not run while in init phase.")
 
@@ -802,6 +890,12 @@ def predict_menu():
         )
 
 
+class MenuEntry(tuple):
+    """
+    The object passed into the choice screen.
+    """
+
+
 def display_menu(items,
                  window_style='menu_window',
                  interact=True,
@@ -882,16 +976,21 @@ def display_menu(items,
 
             if value is not None:
                 action = renpy.ui.ChoiceReturn(label, value, location)
+                chosen = action.get_chosen()
             else:
                 action = None
+                chosen = False
 
             if renpy.config.choice_screen_chosen:
-                if action is not None:
-                    item_actions.append((label, action, action.get_chosen()))
-                else:
-                    item_actions.append((label, action, False))
+                me = MenuEntry((label, action, chosen))
             else:
-                item_actions.append((label, action))
+                me = MenuEntry((label, action))
+
+            me.caption = label
+            me.action = action
+            me.chosen = chosen
+
+            item_actions.append(me)
 
             show_screen(screen, items=item_actions, _widget_properties=props, _transient=True, _layer=renpy.config.choice_layer, **scope)
 
@@ -1124,7 +1223,7 @@ def pause(delay=None, music=None, with_none=None, hard=False, checkpoint=None):
     `checkpoint`
         If true, a checkpoint will be set, and players will be able to roll
         back to this statement. If false, no checkpoint will be set. If None,
-        a checkpoint will only be set if display is set.
+        a checkpoint will only be set if delay is set.
     """
 
 
@@ -1317,7 +1416,7 @@ def rollback(force=False, checkpoints=1, defer=False, greedy=True, label=None):
 
     renpy.config.skipping = None
     renpy.game.log.complete()
-    renpy.game.log.rollback(checkpoints, greedy=greedy, label=label, force=force)
+    renpy.game.log.rollback(checkpoints, greedy=greedy, label=label, force=(force is True))
 
 
 def toggle_fullscreen():
@@ -1416,7 +1515,20 @@ def reload_script():
     save.
     """
 
-    renpy.game.call_in_new_context("_save_reload_game")
+    s = get_screen("menu")
+
+    if not renpy.store.main_menu:
+        if s is not None:
+            session["_reload_screen"] = s.screen_name[0]
+
+        renpy.game.call_in_new_context("_save_reload_game")
+
+    else:
+        if s is not None:
+            session["_main_menu_screen"] = s.screen_name[0]
+
+        utter_restart()
+
 
 
 def quit(relaunch=False, status=0): #@ReservedAssignment
@@ -1491,7 +1603,7 @@ def screenshot(filename):
 @renpy_pure
 def version(tuple=False): #@ReservedAssignment
     """
-    :doc: other
+    :doc: renpy_version
 
     If `tuple` is false, returns a string containing "Ren'Py ", followed by
     the current version of Ren'Py.
@@ -1505,6 +1617,11 @@ def version(tuple=False): #@ReservedAssignment
 
     return renpy.version
 
+version_string = renpy.version
+version_only = renpy.version_only
+version_name = renpy.version_name
+version_tuple = renpy.version_tuple
+license = "" # @ReservedAssignment
 
 def transition(trans, layer=None, always=False, force=False):
     """
@@ -1684,7 +1801,7 @@ def log(msg):
     :doc: debug
 
     If :var:`config.log` is not set, this does nothing. Otherwise, it opens
-    the logfile (if not already open), formats the message to 70
+    the logfile (if not already open), formats the message to :var:`config.log_width`
     columns, and prints it to the logfile.
     """
 
@@ -1707,7 +1824,7 @@ def log(msg):
 
         import textwrap
 
-        print >>logfile, textwrap.fill(msg).encode("utf-8")
+        print >>logfile, textwrap.fill(msg, renpy.config.log_width).encode("utf-8")
         logfile.flush()
 
     except:
@@ -1804,6 +1921,9 @@ def seen_audio(filename):
     Returns True if the given filename has been played at least once on the current
     user's system.
     """
+
+    filename = re.sub(r'^<.*?>', '', filename)
+
     return filename in renpy.game.persistent._seen_audio  # @UndefinedVariable
 
 
@@ -1862,19 +1982,22 @@ def image_size(im):
     return surf.get_size()
 
 
-def get_at_list(name, layer='master'):
+def get_at_list(name, layer=None):
     """
     :doc: se_images
 
     Returns the list of transforms being applied to the image with tag `name`
     on `layer`. Returns an empty list if no transofrms are being applied, or
     None if the image is not shown.
+
+    If `layer` is None, uses the default layer for the given tag.
     """
 
     if isinstance(name, basestring):
         name = tuple(name.split())
 
     tag = name[0]
+    layer = default_layer(layer, tag)
 
     return renpy.game.context().scene_lists.at_list[layer].get(tag, None)
 
@@ -2212,9 +2335,13 @@ def get_image_bounds(tag, width=None, height=None, layer='master'):
     `width`, `height`
         The width and height of the area that contains the image. If None,
         defaults the width and height of the screen, respectively.
+
+    `layer`
+        If None, uses the default layer for `tag`.
     """
 
     tag = tag.split()[0]
+    layer = default_layer(layer, tag)
 
     if width is None:
         width = renpy.config.screen_width
@@ -2577,19 +2704,25 @@ def get_side_image(prefix_tag, image_tag=None, not_showing=True, layer='master')
 
     If not_showing is True, this only returns a side image if the image the
     attributes are taken from is not on the screen.
+
+    If `layer` is None, uses the default layer for the currently showing
+    tag.
     """
 
     images = renpy.game.context().images
 
     if image_tag is not None:
-        attrs = (image_tag,) + images.get_attributes(layer, image_tag)
+        image_layer = default_layer(layer, image_tag)
+        attrs = (image_tag,) + images.get_attributes(image_layer, image_tag)
     else:
         attrs = renpy.store._side_image_attributes
 
     if not attrs:
         return None
 
-    if not_showing and images.showing(layer, (attrs[0], )):
+    attr_layer = default_layer(layer, attrs)
+
+    if not_showing and images.showing(attr_layer, (attrs[0], )):
         return None
 
     required = set()
@@ -2807,13 +2940,21 @@ def count_seen_dialogue_blocks():
 
     Returns the number of dialogue blocks the user has seen in any play-through
     of the current game.
-
-    This number may be larger than the result of :func:`renpy.count_dialogue_blocks`
-    when the script has changed and older dialogue blocks are no longer accessible.
     """
 
     return renpy.game.seen_translates_count
 
+def count_newly_seen_dialogue_blocks():
+    """
+    :doc: other
+
+    Returns the number of dialogue blocks the user has seen for the first time
+    during this session.
+    """
+
+    return renpy.game.new_translates_count
+
+
 
 def substitute(s, scope=None, translate=True):
     """
@@ -2965,3 +3106,184 @@ def clear_line_log():
 
     renpy.game.context().line_log = [ ]
 
+def add_layer(layer, above=None, below=None, menu_clear=True):
+    """
+    :doc: other
+
+    Adds a new layer to the screen. If the layer already exists, this
+    function does nothing.
+
+    One of `behind` or `above` must be given.
+
+    `layer`
+        A string giving the name of the new layer to add.
+
+    `above`
+        If not None, a string giving the name of a layer the new layer will
+        be placed above.
+
+    `below`
+        If not None, a string giving the name of a layer the new layer will
+        be placed below.
+
+    `menu_clear`
+        If true, this layer will be cleared when entering the game menu
+        context, and restored when leaving the
+    """
+
+    layers = renpy.config.layers
+
+    if layer in renpy.config.layers:
+        return
+
+    if (above is not None) and (below is not None):
+        raise Exception("The above and below arguments to renpy.add_layer are mutually exclusive.")
+
+    elif above is not None:
+        try:
+            index = layers.index(above) + 1
+        except ValueError:
+            raise Exception("Layer '%s' does not exist." % above)
+
+    elif below is not None:
+        try:
+            index = layers.index(below)
+        except ValueError:
+            raise Exception("Layer '%s' does not exist." % below)
+
+    else:
+        raise Exception("The renpy.add_layer function requires either the above or below argument.")
+
+    layers.insert(index, layer)
+
+    if menu_clear:
+        renpy.config.menu_clear_layers.append(layer) # @UndefinedVariable
+
+
+def maximum_framerate(t):
+    """
+    :doc: other
+
+    Forces Ren'Py to draw the screen at the maximum framerate for `t` seconds.
+    If `t` is None, cancels the maximum framerate request.
+    """
+
+    if renpy.display.interface is not None:
+        renpy.display.interface.maximum_framerate(t)
+    else:
+        if t is None:
+            renpy.display.core.initial_maximum_framerate = 0
+        else:
+            renpy.display.core.initial_maximum_framerate = max(renpy.display.core.initial_maximum_framerate, t)
+
+
+def is_start_interact():
+    """
+    :doc: other
+
+    Returns true if restart_interaction has not been called during the current
+    interaction. This can be used to determine if the interaction is just being
+    started, or has been restarted.
+    """
+
+    return renpy.display.interface.start_interact
+
+
+def play(filename, channel=None, **kwargs):
+    """
+    :doc: audio
+
+    Plays a sound effect. If `channel` is None, it defaults to
+    :var:`config.play_channel`. This is used to play sounds defined in
+    styles, :propref:`hover_sound` and :propref:`activate_sound`.
+    """
+
+    if filename is None:
+        return
+
+    if channel is None:
+        channel = renpy.config.play_channel
+
+    renpy.audio.music.play(filename, channel=channel, **kwargs)
+
+
+def get_editable_input_value():
+    """
+    :undocumented:
+
+    Returns the current input value, and a flag that is true if it is editable.
+    and false otherwise.
+    """
+
+    return renpy.display.behavior.current_input_value, renpy.display.behavior.input_value_active
+
+
+def set_editable_input_value(input_value, editable):
+    """
+    :undocumented:
+
+    Sets the currently active input value, and if it should be marked as
+    editable.
+    """
+
+    renpy.display.behavior.current_input_value = input_value
+    renpy.display.behavior.input_value_active = editable
+
+
+def get_refresh_rate(precision=5):
+    """
+    :doc: other
+
+    Returns the refresh rate of the current screen, as a floating-point
+    number of frames per second.
+
+    `precision`
+        The raw data Ren'Py gets is number of frames per second, rounded down.
+        This means that a monitor that runs at 59.95 frames per second will
+        be reported at 59 fps. The precision argument reduces the precision
+        of this reading, such that the only valid readings are multiples of
+        the precision.
+
+        Since all monitor framerates tend to be multiples of 5 (25, 30, 60,
+        75, and 120), this likely will improve accuracy. Setting precision
+        to 1 disables this.
+    """
+
+    precision *= 1.0
+
+    info = renpy.display.get_info()
+    rv = info.refresh_rate
+    rv = round(rv / precision) * precision
+
+    return rv
+
+
+def get_identifier_checkpoints(identifier):
+    """
+    :doc: rollback
+
+    Given a rollback_identifier from a HistoryEntry object, returns the number
+    of checkpoints that need to be passed to :func:`renpy.rollback` to reach
+    that identifier. Returns None of the identifier is not in the rollback
+    history.
+    """
+
+    return renpy.game.log.get_identifier_checkpoints(identifier)
+
+def get_adjustment(bar_value):
+    """
+    :doc: other
+
+    Given `bar_value`, a  :class:`BarValue`, returns the :func:`ui.adjustment`
+    if uses. The adjustment has the following to attributes defined:
+
+    .. attribute:: value
+
+        The current value of the bar.
+
+    .. attribute:: range
+
+        The current range of the bar.
+    """
+
+    return bar_value.get_adjustment()
